@@ -1,14 +1,17 @@
 import turtle;
+import tkinter as tk;
 import inspect;
 import time;
 
 from pydraw import Color;
 from pydraw import Location;
+from pydraw.errors import *;
 
 INPUT_TYPES = [
     'mousedown',
     'mouseup',
     'mousedrag',
+    'mousemove',
     'keydown',
     'keyup',
     'keypress'
@@ -113,6 +116,12 @@ class Screen:
 
     def __init__(self, width=800, height=600, title="pydraw"):
         self._screen = turtle.Screen();
+        self._canvas = self._screen.cv;
+        self._root = self._canvas.winfo_toplevel();
+
+        # The only thing on the canvas is itself, so we prevent anything stupid from happening.
+        self._canvas.configure(scrollregion=self._canvas.bbox("all"));
+
         self._screen.screensize(width, height);
         self._screen.setup(width + BORDER_CONSTANT, height + BORDER_CONSTANT);
         # This was not necessary as the canvas will align with the window's dimensions as set in the above line.
@@ -122,10 +131,18 @@ class Screen:
 
         self._screen.colormode(255);
 
+        # store the mouse position
+        self._mouse = Location(0, 0);
+        self._gridlines = [];
+        self._gridstate = False;  # grid is disabled by default
+
         # By default we want to make sure that all objects are drawn instantly.
         self._screen.tracer(0);
         self._screen.update();
 
+        # self._root.protocol('WM_DELETE_WINDOW', self._exit_handler);
+        # atexit.register(self._exit_handler)
+
         # --- #
 
         self.registry = {};  # The input function registry (stores input callbacks)
@@ -149,6 +166,7 @@ class Screen:
         :param color: the color to set the background to
         :return: None
         """
+
         self._screen.bgcolor(color.__value__());
 
     def picture(self, pic: str) -> None:
@@ -157,9 +175,9 @@ class Screen:
         :param pic: the path to said picture from the file
         :return: None
         """
+
         self._screen.bgpic(pic);
 
-    # Size
     def resize(self, width, height) -> None:
         """
         Resize the screen to new dimensions
@@ -167,15 +185,25 @@ class Screen:
         :param height: the height to resize to
         :return: None
         """
-        self._screen.screensize(width, height);
+
+        # noinspection PyBroadException
+        try:
+            self._screen.screensize(width, height);
+        except:
+            pass;
 
     def size(self) -> (int, int):
         """
-        Get the size of the screen (please note this is not the canvas, and those attributes should be
+        Get the size of the WINDOW (please note this is not the canvas, and those attributes should be
         retrieved using the width() and height() methods respectively)
-        :return: a tuple containing the width and height of the screen
+        :return: a tuple containing the width and height of the WINDOW
         """
-        return self._screen.window_width(), self._screen.window_height();
+
+        # noinspection PyBroadException
+        try:
+            return self._screen.window_width(), self._screen.window_height();
+        except:
+            return -1, -1;  # Again, trying to avoid showing errors due to tkinter shutting down.
 
     def width(self) -> int:
         """
@@ -201,7 +229,87 @@ class Screen:
         except:
             return -1;  # Just return -1 because tkinter is shutting down
 
+    def mouse(self) -> Location:
+        """
+        Get the current mouse-position
+        :return: the mouse-position in the form of a Location
+        """
+
+        return self._mouse;
+
     # Direct Manipulation
+    def prompt(self, text: str, title: str = 'Prompt') -> str:
+        """
+        Prompts the user for keyboard input
+        :param: text the text to prompt the user with
+        :param: title the title of the dialog box
+        :return: None
+        """
+
+        text = self._screen.textinput(title, text);
+
+        self._screen.listen();  # keep us nice and listening :)
+        return text;
+
+    def grid(self, rows: int = None, cols: int = None, cellsize: tuple = (50, 50)):
+        from pydraw import Line;
+
+        if len(self._gridlines) > 0:
+            [line.remove() for line in self._gridlines];
+            self._gridlines.clear();
+        self._gridstate = True;
+
+        if rows is not None:
+            cellsize = (self.height() / rows, cellsize[1]);
+        if cols is not None:
+            cellsize = (cellsize[0], self.width() / cols);
+
+        for row in range(int(cellsize[1]), int(self.height()), int(cellsize[1])):
+            self._gridlines.append(Line(self, Location(0, row), Location(self.width(), row),
+                                        color=Color('lightgray')));
+
+        for col in range(int(cellsize[0]), int(self.width()), int(cellsize[0])):
+            self._gridlines.append(Line(self, Location(col, 0), Location(col, self.height()),
+                                        color=Color('lightgray')));
+
+    def toggle_grid(self, value=None):
+        if value is None:
+            value = not self._gridstate;
+
+        if len(self._gridlines) == 0:
+            self.grid();  # Create a grid if one does not exist.
+
+        [line.visible(value) for line in self._gridlines];
+
+    def grab(self, filename: str = None) -> str:
+        """
+        Grabs a screenshot of the image and saves it to the directory with the specified filename!
+        Note that if no filename is specified the file will be given a name based on the epoch time.
+        :param filename: the name of the file to save the screenshot to.
+        :return: the name of the file.
+        """
+
+        # noinspection PyBroadException
+        try:
+            from PIL import ImageGrab;
+
+            # We need to get the exact canvas coordinates. (bruh)
+            x1 = self._root.winfo_rootx() + self._canvas.winfo_x() + BORDER_CONSTANT;
+            y1 = self._root.winfo_rooty() + self._canvas.winfo_y() + BORDER_CONSTANT;
+            x2 = x1 + self.width() - BORDER_CONSTANT;
+            y2 = y1 + self.height() - BORDER_CONSTANT;
+
+            if filename is None:
+                filename = 'pydraw' + str(time.time() % 10000);
+
+            if not filename.endswith('.png'):
+                filename += '.png';
+
+            ImageGrab.grab().crop((x1, y1, x2, y2)).save(filename);
+            return filename;
+        except:
+            raise UnsupportedError('As PIL is not installed, you cannot grab the screen! ' 
+                                   'Install Pillow via: \'pip install pillow\'.');
 
     # noinspection PyProtectedMember
     def remove(self, obj):
@@ -214,16 +322,23 @@ class Screen:
             self._screen.cv.delete(obj._ref);
             del obj;
 
+    # noinspection PyProtectedMember
+    def front(self, obj):
+        from pydraw import Renderable, CustomRenderable;
+        if isinstance(obj, Renderable) and not isinstance(obj, CustomRenderable):
+            obj._ref.forward(0);
+        else:
+            self._canvas.tag_raise(obj._ref);
+
     def clear(self) -> None:
         """
         Clears the screen.
         :return: None
         """
 
-        # noinspection PyBroadException
         try:
             self._screen.clear();
-        except:
+        except (tk.TclError, AttributeError):
             pass;  # We silently stop TclErrors from appearing to users.
 
     @staticmethod
@@ -243,7 +358,7 @@ class Screen:
         """
         try:
             self._screen.update();
-        except turtle.Terminator:
+        except (turtle.Terminator, tk.TclError, AttributeError):
             # If we experience the termination exception, we will print the termination of the program
             # and exit the python program.
             print('Terminated.');
@@ -257,6 +372,22 @@ class Screen:
         self._screen.clear();
         turtle.done();
 
+    def _colorstr(self, color: Color) -> str:
+        """
+        Takes a pydraw Color and returns a tkinter-friendly string, while also preventing errors
+        from occurring after tkinter has shut down.
+        :param color: the Color to convert
+        :return: the converted color (tkinter-str)
+        """
+
+        try:
+            # noinspection PyProtectedMember
+            # noinspection PyUnresolvedReferences
+            colorstr = self._screen._colorstr(color.__value__());
+            return colorstr;
+        except (turtle.TurtleGraphicsError, tk.TclError):
+            pass;
+
     # ------------------------------------------------------- #
 
     def listen(self) -> None:
@@ -281,7 +412,7 @@ class Screen:
                 continue;
 
             self.registry[name.lower()] = function;
-            print('Registered input-function:', name);
+            # print('Registered input-function:', name);
 
         self._listen();
 
@@ -305,6 +436,8 @@ class Screen:
             # custom implemented mouseclick
             self._onmouseclick(self._create_lambda('mouseclick', btn), btn);
 
+        self._screen.cv.bind("<Motion>", (self._create_lambda('mousemove', None)))
+
     class Key:
         def __init__(self, key: str):
             self._key = key;
@@ -358,6 +491,8 @@ class Screen:
             return lambda x, y: (self._mouseup(key, self.create_location(x, y)));
         elif method == 'mousedrag':
             return lambda x, y: (self._mousedrag(key, self.create_location(x, y)));
+        elif method == 'mousemove':
+            return lambda event: (self._mousemove(Location(event.x, event.y)));
         else:
             return None;
 
@@ -403,6 +538,15 @@ class Screen:
 
         self.registry['mousedrag'](button, location);
 
+    def _mousemove(self, location) -> None:
+        # We will update our internal storage of the mouse-location no matter what
+        self._mouse = location;
+
+        if 'mousemove' not in self.registry:
+            return;
+
+        self.registry['mousemove'](location);
+
     # --- Helper Methods --- #
     def create_location(self, x, y) -> Location:
         """
