@@ -17,7 +17,7 @@ import time;
 from pydraw import Screen, Color, Location, Rectangle, Oval, Triangle, Text, Polygon, CustomPolygon, Image, Line;
 
 screen = Screen(800, 600, 'Something Awesome!');
-screen.color(Color('black'));
+screen.color(Color('white'));
 
 noah = Triangle(screen, 23, 45, 35, 60, Color('green'));
 noah.rotation(90);
@@ -45,32 +45,58 @@ text = Text(screen, 'barry', 0, 0, color=Color('white'), italic=True, bold=True)
 print(f'Text Width: {text.width()}, Text Height: {text.height()}')
 text.color(Color(155, 155, 155));
 text.rotation(32);
+text.move(dx=25, dy=25);
+text.moveto(x=300);
 
 barry.overlaps(noah);
 
 waffle = Polygon(screen, 6, screen.width() - (screen.width() / 3), 100, 50, 50, color=Color('blue'));
-
-# tester = Rectangle(screen, 50, 350, 50, 50, Color('purple'));
-
+waffle.border(Color('red'));
+waffle.border(Color.NONE);
 crazy_waffle = CustomPolygon(screen, [(50, 350), (100, 350), (100, 400), (75, 450), (50, 400)], color=Color('green'));
 
 maui = Line(screen, 50, 50, 150, 150, color=Color('magenta'));
-mrspace = Image(screen, 'featuredspace_logo.png', screen.width() / 2, screen.height() / 2, 150, 150);
-mrspace.move(-mrspace.width() / 2, -mrspace.height() / 2);
+moana = maui.clone();
+moana.rotation(45);
+moana.color(Color('gray17'));
 
-#
-# print(f'Turtle Pos: {crazy_waffle._ref.pos()}');
-# print(f'Pydraw Pos: {crazy_waffle.location()}');
-# print(f'Supposed Location: {crazy_waffle._get_real_location()}')
+maui.moveto(pos1=Location(200, 100));
+
+mrspace = Image(screen, 'featuredspace_logo.png', screen.width() / 2, screen.height() / 2);
+mrspace.width(50);
+mrspace.height(50)
+
+mrspace.move(-mrspace.width() / 2, -mrspace.height() / 2);
+mrspace.color(Color('red'))
+
+cool_barry = Image(screen, 'cool_barry.jpg', screen.width() / 3, screen.height() / 2, 150, 150);
+
+print(f'Screen Size: {screen.size()}')
+
+mrspace.rotation(45)
+# mrspace.border(Color('green'));
 
 state = True;
 
+test_color = Color('red2');
+print(f'Color RGB: {test_color.red()}, {test_color.green()}, {test_color.blue()}');
+
 
 def mousedown(button, location):
     global state;
     print('Mousedown detected', button, location);
     crazy_waffle.rotate(5 if state else -5);
 
+    if button == 3:
+        print('grabbed screen');
+        screen.grab('test.png');
+    elif button == 2:
+        print('cloning barry');
+        barry2 = barry.clone();
+        barry2.move(-32, -32);
+    elif location.x() > screen.width() / 2:
+        waffle.transform(barry.transform());
+
     state = not state;
 
 
@@ -100,6 +126,8 @@ screen.listen();
 
 running = True;
 while running:
+    mrspace.rotate(5);
+    maui.lookat(barry.center());
     screen.update();
     screen.sleep(30 / 1000);
 
